# File S6_Calculation.r
# Input: Table S1_Dataset including metabolite data and metadata.xlsx
# Output: File S7_Results.rds

library("openxlsx")
library("missForest")
library("caret")

karmen.data<-read.xlsx(xlsxFile="Table S1_Dataset including metabolite data and metadata.xlsx",sheet="Data",colNames=FALSE)
karmen.anno<-read.xlsx(xlsxFile="Table S1_Dataset including metabolite data and metadata.xlsx",sheet="Annotation",colNames=FALSE)
colnames(karmen.data)[1:5]<-c("ID","VO2peak","sex","age","mp")
colnames(karmen.anno)[1:5]<-c("variable","VO2peak","sex","age","mp")

# proband id
karmen.data<-karmen.data[-1,]
karmen.data$ID<-c(paste("P00",1:9,sep=""),paste("P0",10:99,sep=""),paste("P",100:(dim(karmen.data)[1]),sep=""))

# variable name sets
tar<-colnames(karmen.anno)[2]
con<-colnames(karmen.anno)[3:5]
klv<-colnames(karmen.anno)[c(7,9:28)]
met<-colnames(karmen.anno)[29:dim(karmen.anno)[2]]

data<-karmen.data[,c("ID",tar,con,klv)]
data$Typ<-data$mp
data$Typ[data$sex==1]<-0
for(indnam in setdiff(colnames(data),c("ID","sex","mp","Typ"))){
  data[,indnam]<-as.numeric(data[,indnam])
}
for(indnam in c("ID","sex","mp","Typ")){
  data[,indnam]<-as.factor(data[,indnam])
}

# Random Forest imputation in AH_MW_AEE_AF, PWV and VdW-scores
ipv<-klv[apply(data[,klv],2,function(arg) sum(is.na(arg)))>0]
dataakt<-data[,setdiff(colnames(data),c("ID",tar,"sex","mp"))]
set.seed(88)
dataakt<-missForest(dataakt,ntree=1000)[[1]]
data[,ipv]<-dataakt[,ipv]

dat.li<-list(data[data$sex==2,],data[data$sex==2,],data[data$sex==2,],data[data$sex==2,],
             data[data$sex==1,],data[data$sex==1,],data[data$sex==1,],data[data$sex==1,])
names(dat.li)<-c("F.non","F.con","F.all","F.ori","M.non","M.con","M.all","M.ori")

for(datnam in c("F.non","M.non")){
  for(indnam in setdiff(colnames(data),c("ID","sex","mp","Typ"))){
    dat.li[[datnam]][,indnam]<-qnorm(rank(dat.li[[datnam]][,indnam],na.last="keep")/(1+length(dat.li[[datnam]][,indnam])))
  }
}

# n.d. and <LOQ imputation and VdW-scores
met.li<-list(karmen.data[karmen.data$sex==2,c("ID",met)],
             karmen.data[karmen.data$sex==2,c("ID",met)],
             karmen.data[karmen.data$sex==2,c("ID",met)],
             karmen.data[karmen.data$sex==2,c("ID",met)],
             karmen.data[karmen.data$sex==1,c("ID",met)],
             karmen.data[karmen.data$sex==1,c("ID",met)],
             karmen.data[karmen.data$sex==1,c("ID",met)],
             karmen.data[karmen.data$sex==1,c("ID",met)])
names(met.li)<-c("F.non","F.con","F.all","F.ori","M.non","M.con","M.all","M.ori")
for(indnam in met){
  akt=karmen.data[,indnam]
  lod<-as.numeric(karmen.anno[karmen.anno$variable=="LOD",indnam])
  loq<-as.numeric(karmen.anno[karmen.anno$variable=="LOQ",indnam])
  xy.n.d.<-akt=="n.d."
  xy.LOQ <-akt=="<LOQ"
  akt<-as.numeric(akt)
  xy.na2<-is.na(akt)
  mini<-min(akt[!xy.na2&akt>0])
  if( is.na(lod)){lod<-mini}
  if(!is.na(loq)){lod<-min(loq,lod)}
  dataakt<-data.frame(ID=karmen.data[,"ID"],sex=karmen.data[,"sex"],met.lod=akt,cen.lod=NA)
  dataakt$cen.lod[akt >lod]<-1
  dataakt$cen.lod[akt==lod]<-1
  if(sum(akt==lod,na.rm=TRUE)>1){
    dataakt$cen.lod[akt==lod]<-0
    dataakt$met.lod[akt==lod]<-mini/4
  }
  dataakt$cen.lod[akt<lod]<-0
  dataakt$met.lod[akt<lod]<-mini/4
  dataakt$met.lod[xy.n.d.]<-mini/4
  dataakt$cen.lod[xy.n.d.]<-0
  dataakt$met.lod[xy.LOQ ]<-mini/2
  dataakt$cen.lod[xy.LOQ ]<-0
  dataakt.F<-dataakt[dataakt$sex==2,]
  dataakt.M<-dataakt[dataakt$sex==1,]
  met.li[["F.ori"]][,indnam]<-dataakt.F$met.lod
  met.li[["M.ori"]][,indnam]<-dataakt.M$met.lod
  met.li[["F.non"]][,indnam]<-qnorm(rank(dataakt.F$met.lod,na.last="keep")/(1+length(dataakt.F$met.lod[!is.na(dataakt.F$met.lod)])))
  met.li[["M.non"]][,indnam]<-qnorm(rank(dataakt.M$met.lod,na.last="keep")/(1+length(dataakt.M$met.lod[!is.na(dataakt.M$met.lod)])))
}

# confounder adjusting
for(indnam in setdiff(colnames(data),c("ID","sex","mp","Typ"))){
  dat.li$F.con[,indnam]<-resid(lm(get(indnam)~Typ+age,data=dat.li$F.non,na.action=na.exclude))
  dat.li$M.con[,indnam]<-resid(lm(get(indnam)~age,data=dat.li$M.non,na.action=na.exclude))
}
dat.li$F.all<-dat.li$F.non
dat.li$M.all<-dat.li$M.non
for.klv<-paste("age+",paste(klv,collapse="+"),sep="")
indnam<-tar
dat.li$F.all[,indnam]<-resid(lm(as.formula(paste(indnam,"~Typ+",for.klv,sep="")),data=dat.li$F.non,na.action=na.exclude))
dat.li$M.all[,indnam]<-resid(lm(as.formula(paste(indnam,"~",for.klv,sep="")),data=dat.li$M.non,na.action=na.exclude))

for(indnam in met){
  met.li$F.con[,indnam]<-resid(lm(met.li$F.non[,indnam]~Typ+age,data=dat.li$F.non,na.action=na.exclude))
  met.li$M.con[,indnam]<-resid(lm(met.li$M.non[,indnam]~    age,data=dat.li$M.non,na.action=na.exclude))
  met.li$F.all[,indnam]<-resid(lm(as.formula(paste(parse(text="met.li$F.non[,indnam]"),"~Typ+",for.klv,sep="")),
                                  data=dat.li$F.non,na.action=na.exclude))
  met.li$M.all[,indnam]<-resid(lm(as.formula(paste(parse(text="met.li$M.non[,indnam]"),"~",for.klv,sep="")),
                                  data=dat.li$M.non,na.action=na.exclude))
}

#####################################################################################################################
#####################################################################################################################
#####################################################################################################################
# correlation
nam<-klv
cor.klch<-NULL
for(datnam in c("F.non","M.non","F.con","M.con")){
  akt.res<-data.frame(name=nam,
                      lname=as.character(karmen.anno[karmen.anno$variable=="Variable",nam]),
                      sname=as.character(karmen.anno[karmen.anno$variable=="Abbreviation",nam]),
                      vargr="klv",pop=substr(datnam,1,1),adj=substr(datnam,3,5),PC=NA,pv=NA,lo=NA,up=NA)
  for(indnam in nam){
    t1<-cor.test(x=dat.li[[datnam]][,tar],y=dat.li[[datnam]][,indnam],method=c("pearson"))
    akt.res$PC[akt.res$name==indnam]=as.numeric(t1$estimate)
    akt.res$pv[akt.res$name==indnam]=t1$p.value
    akt.res$lo[akt.res$name==indnam]=t1$conf.int[1]
    akt.res$up[akt.res$name==indnam]=t1$conf.int[2]
  }
  cor.klch<-rbind(cor.klch,akt.res)
}
cor.klch<-cor.klch[order(cor.klch$PC),]
cor.klch$pop<-factor(cor.klch$pop,levels=c("F","M"))
cor.klch$adj<-factor(cor.klch$adj,levels=c("non","con","all"))

nam<-met
cor.meta<-NULL
for(datnam in c("F.non","M.non","F.con","M.con","F.all","M.all")){
  akt.res<-data.frame(name=nam,
                      lname=as.character(karmen.anno[karmen.anno$variable=="Variable",nam]),
                      sname=as.character(karmen.anno[karmen.anno$variable=="Abbreviation",nam]),
                      Major.Pathway.Class=as.character(karmen.anno[karmen.anno$variable=="Major pathway/class",nam]),
                      Specific.Pathway.Class=as.character(karmen.anno[karmen.anno$variable=="Specific pathway/class",nam]),
                      HMDB.ID=as.character(karmen.anno[karmen.anno$variable=="HMDB ID",nam]),
                      KEGG.ID=as.character(karmen.anno[karmen.anno$variable=="KEGG ID",nam]),
                      vargr="met",pop=substr(datnam,1,1),adj=substr(datnam,3,5),PC=NA,pv=NA,lo=NA,up=NA)
  for(indnam in nam){
    t1<-cor.test(x=dat.li[[datnam]][,tar],y=met.li[[datnam]][,indnam],method=c("pearson"))
    akt.res$PC[akt.res$name==indnam]=as.numeric(t1$estimate)
    akt.res$pv[akt.res$name==indnam]=t1$p.value
    akt.res$lo[akt.res$name==indnam]=t1$conf.int[1]
    akt.res$up[akt.res$name==indnam]=t1$conf.int[2]
  }
  cor.meta<-rbind(cor.meta,akt.res)
}
cor.meta$pop<-factor(cor.meta$pop,levels=c("F","M"))
cor.meta$adj<-factor(cor.meta$adj,levels=c("non","con","all"))

#####################################################################################################################
#####################################################################################################################
#####################################################################################################################
# PLS Rank-Product-Permutation-Tests

# Pre process functions (VdW-scaling, adjusting)
funpreproc.F.con<-function(data,id.te,id.tr){
    datatr<-data[data$ID%in%id.tr,c("ID","age","Typ","y",met)]
    datate<-data[data$ID%in%id.te,c("ID","age","Typ","y",met)]
    for(indnam in c("y","age",met)){
      datate[,indnam]<-qnorm(sapply(datate[,indnam],FUN=function(arg) sum(datatr[,indnam]<arg)+(1+sum(datatr[,indnam]==arg))/2)/
                               (1+length(id.tr)))
      datatr[,indnam]<-qnorm(rank(datatr[,indnam],na.last="keep")/(1+length(id.tr)))
    }
    for(indnam in c("y",met)){
      modlm<-lm(get(indnam)~Typ+age,data=datatr)
      datatr[,indnam]<-resid(modlm)
      datate[,indnam]<-datate[,indnam]-predict(modlm,newdata=datate)
    }
    return(list(x.tr=datatr[,met],y.tr=datatr[,"y"],x.te=datate[,met],y.te=datate[,"y"]))
  }
funpreproc.M.con<-function(data,id.te,id.tr){
  datatr<-data[data$ID%in%id.tr,c("ID","age","y",met)]
  datate<-data[data$ID%in%id.te,c("ID","age","y",met)]
  for(indnam in c("y","age",met)){
    datate[,indnam]<-qnorm(sapply(datate[,indnam],FUN=function(arg) sum(datatr[,indnam]<arg)+(1+sum(datatr[,indnam]==arg))/2)/
                             (1+length(id.tr)))
    datatr[,indnam]<-qnorm(rank(datatr[,indnam],na.last="keep")/(1+length(id.tr)))
  }
  for(indnam in c("y",met)){
    modlm<-lm(get(indnam)~age,data=datatr)
    datatr[,indnam]<-resid(modlm)
    datate[,indnam]<-datate[,indnam]-predict(modlm,newdata=datate)
  }
  return(list(x.tr=datatr[,met],y.tr=datatr[,"y"],x.te=datate[,met],y.te=datate[,"y"]))
}
funpreproc.F.all<-function(data,id.te,id.tr){
  datatr<-data[data$ID%in%id.tr,]
  datate<-data[data$ID%in%id.te,]
  for(indnam in c("y","age",klv,met)){
    datate[,indnam]<-qnorm(sapply(datate[,indnam],FUN=function(arg) sum(datatr[,indnam]<arg)+(1+sum(datatr[,indnam]==arg))/2)/
                             (1+length(id.tr)))
    datatr[,indnam]<-qnorm(rank(datatr[,indnam],na.last="keep")/(1+length(id.tr)))
  }
  for(indnam in c("y",met)){
    modlm<-lm(get(indnam)~Typ+age+X7+X9+X10+X11+X12+X13+X14+X15+X16+X17+X18+X19+X20+X21+X22+X23+X24+X25+X26+X27+X28,
              data=datatr)
    datatr[,indnam]<-resid(modlm)
    datate[,indnam]<-datate[,indnam]-predict(modlm,newdata=datate)
  }
  return(list(x.tr=datatr[,met],y.tr=datatr[,"y"],x.te=datate[,met],y.te=datate[,"y"]))
}
funpreproc.M.all<-function(data,id.te,id.tr){
  datatr<-data[data$ID%in%id.tr,]
  datate<-data[data$ID%in%id.te,]
  for(indnam in c("y","age",klv,met)){
    datate[,indnam]<-qnorm(sapply(datate[,indnam],FUN=function(arg) sum(datatr[,indnam]<arg)+(1+sum(datatr[,indnam]==arg))/2)/
                             (1+length(id.tr)))
    datatr[,indnam]<-qnorm(rank(datatr[,indnam],na.last="keep")/(1+length(id.tr)))
  }
  for(indnam in c("y",met)){
    modlm<-lm(get(indnam)~age+X7+X9+X10+X11+X12+X13+X14+X15+X16+X17+X18+X19+X20+X21+X22+X23+X24+X25+X26+X27+X28,
              data=datatr)
    datatr[,indnam]<-resid(modlm)
    datate[,indnam]<-datate[,indnam]-predict(modlm,newdata=datate)
  }
  return(list(x.tr=datatr[,met],y.tr=datatr[,"y"],x.te=datate[,met],y.te=datate[,"y"]))
}
funpreproc.li<-list("F.con"=funpreproc.F.con,"F.all"=funpreproc.F.all,
                    "M.con"=funpreproc.M.con,"M.all"=funpreproc.M.all)

#####################################################################################################################
splitrep<-20
te.ant<-0.2
foldin<-8
resendnam<-c("popadj","perm.nr","seed","test.nr","n.te","n.tr",
             "nc1","nc2","nc3","nc4","nc5","nc6","nc7","nc8","nc9","nc.g9",
             "RMSE.mvcv","R2.mvcv","MAE.mvcv","RMSE.sdcv","R2.sdcv","MAE.sdcv",
             "RMSE.mvte","R2.mvte","MAE.mvte","RMSE.sdte","R2.sdte","MAE.sdte",
             met)
resend<-as.data.frame(matrix(ncol=length(resendnam),nrow=1))
names(resend)<-resendnam
reszwinam<-c("test.nr",
             "ncomp","RMSE","Rsquared","MAE",
             "RMSE.te","Rsquared.te","MAE.te",met)
resnam1<-c("ncomp","RMSE","Rsquared","MAE")
reszwi<-as.data.frame(matrix(ncol=length(reszwinam),nrow=splitrep))
names(reszwi)<-reszwinam
resakt<-as.data.frame(matrix(ncol=length(reszwinam),nrow=1))
names(resakt)<-reszwinam
ctrl<-trainControl(method="repeatedcv",number=foldin,repeats=1)

seedv<-0
permset.li<-list(c(0,1:100),c(0,101:300),c(0,301:500),c(0,501:700),c(0,701:900),901:1000,
                 1001:1100,1101:1200,1201:1300,1301:1400,1401:1500,
                 1501:1600,1601:1700,1701:1800,1801:1900,1901:2000,
                 2001:2100,2101:2200,2201:2300,2301:2400,2401:2500)
pls.res<-NULL
#####################################################################################################################
# following computation need some time
#####################################################################################################################
for(permsetind in 1:length(permset.li)){
  permset<-permset.li[[permsetind]]
  result<-NULL
  seedv<-seedv+1000000
  for(popadj in c("F.con","F.all","M.con","M.all")){
    data<-merge(dat.li[[popadj]],met.li[[popadj]],by="ID")
    data$Typ<-as.character(data$Typ)
    data$y<-data[,tar]
    set.seed(seedv)
    permnr<-0
    for(permnr in permset){
      if(permnr!=0){
        data$y[data$Typ==0]<-sample(data[data$Typ==0,tar])
        data$y[data$Typ==1]<-sample(data[data$Typ==1,tar])
        data$y[data$Typ==2]<-sample(data[data$Typ==2,tar])
      }
      te.li<-createDataPartition(data$Typ,p=te.ant,times=splitrep)
      for(wdh in 1:splitrep){
        te.akt<-as.character(data$ID[te.li[[wdh]]])
        tr.akt<-setdiff(as.character(data$ID),te.akt)
        resakt$test.nr<-wdh
        akt.li<-funpreproc.li[[popadj]](data,te.akt,tr.akt)
        mod<-train(x=akt.li[["x.tr"]],y=akt.li[["y.tr"]],
                   method="pls",preProc=c("center","scale"),
                   trControl=ctrl,tuneGrid=expand.grid(ncomp=1:10))
        resakt[1,resnam1]<-mod[["results"]][mod[["results"]]$ncomp==mod[["finalModel"]]$ncomp,resnam1]
        rg<-rank(-abs(coef(mod[["finalModel"]])))
        names(rg)<-attr(coef(mod[["finalModel"]]),"dimnames")[[1]]
        resakt[1,names(rg)]<-rg
        resakt[1,c("RMSE.te","Rsquared.te","MAE.te")]<-postResample(predict(mod,newdata=akt.li[["x.te"]]),
                                                                    akt.li[["y.te"]])
        reszwi[wdh,]<-resakt[which.min(resakt$RMSE),]
      }
      resend[1,"popadj"]=popadj
      resend[1,"perm.nr"]=permnr
      resend[1,"seed"]=seedv
      resend[1,"test.nr"]=splitrep
      resend[1,"n.te"]=length(te.akt)
      resend[1,"n.tr"]=length(tr.akt)
      resend[1,"nc1"]=sum(reszwi$ncomp==1)
      resend[1,"nc2"]=sum(reszwi$ncomp==2)
      resend[1,"nc3"]=sum(reszwi$ncomp==3)
      resend[1,"nc4"]=sum(reszwi$ncomp==4)
      resend[1,"nc5"]=sum(reszwi$ncomp==5)
      resend[1,"nc6"]=sum(reszwi$ncomp==6)
      resend[1,"nc7"]=sum(reszwi$ncomp==7)
      resend[1,"nc8"]=sum(reszwi$ncomp==8)
      resend[1,"nc9"]=sum(reszwi$ncomp==9)
      resend[1,"nc.g9"]=sum(reszwi$ncomp>9)
      resend[1,c("RMSE.mvcv","R2.mvcv","MAE.mvcv","RMSE.mvte","R2.mvte","MAE.mvte")
      ]=apply(reszwi[,c("RMSE","Rsquared","MAE","RMSE.te","Rsquared.te","MAE.te")],2,mean)
      resend[1,c("RMSE.sdcv","R2.sdcv","MAE.sdcv","RMSE.sdte","R2.sdte","MAE.sdte")
      ]=apply(reszwi[,c("RMSE","Rsquared","MAE","RMSE.te","Rsquared.te","MAE.te")],2,sd)
      resend[1,met]<-apply(log(reszwi[,met]),2,mean)
      result<-rbind(result,resend)
    }
  }
  pls.res<-rbind(pls.res,result)
}

popadjnam<-c("F.all"="F**","F.con"="F*","M.all"="M**","M.con"="M*")
pls.perm<-NULL
for(popadj in c("F.con","F.all","M.con","M.all")){
  datazwi<-data.frame(name=met,pop=substr(popadj,1,1),adj=substr(popadj,3,5),gm.ori=NA,gm.per=NA,rg.ori=NA,rg.per=NA,rg.pv=NA)
  for(indnam in met){
    datazwi$gm.ori[datazwi$name==indnam]<-mean(pls.res[pls.res$popadj==popadj&pls.res$perm.nr==0,indnam])
    datazwi$gm.per[datazwi$name==indnam]<-mean(pls.res[pls.res$popadj==popadj&pls.res$perm.nr>0,indnam])
    datazwi$rg.ori[datazwi$name==indnam]<-exp(datazwi$gm.ori[datazwi$name==indnam])
    datazwi$rg.per[datazwi$name==indnam]<-exp(datazwi$gm.per[datazwi$name==indnam])
    datazwi$rg.pv[datazwi$name==indnam]<-sum(datazwi$gm.ori[datazwi$name==indnam]>pls.res[pls.res$popadj==popadj&pls.res$perm.nr>0,indnam])/
      sum(pls.res$perm.nr[pls.res$popadj==popadj]>0)
  }
  pls.perm<-rbind(pls.perm,datazwi)
}
pls.perm<-merge(cor.meta,pls.perm,by=c("name","pop","adj"))
pls.vali<-pls.res[,c("popadj","perm.nr","RMSE.mvte")]
#####################################################################################################################
#####################################################################################################################
#####################################################################################################################
# multiple linear regression

# forward selection
myli<-list(list(klv,met),list(NULL,c(klv,met)))
starttime<-Sys.time()
res.li<-list()
maxanz<-80
pop<-"F"
index<-1
for(pop in c("F","M")){
  dataakt<-merge(dat.li[[paste(pop,".con",sep="")]],met.li[[paste(pop,".con",sep="")]],by="ID")
  n<-dim(dataakt)[1]
  tss<-(n-1)*var(dataakt[,tar])
  for(index in 1:length(myli)){
    varfix<-myli[[index]][[1]]
    varsel<-myli[[index]][[2]]
    akt.for<-paste(tar,"~1",sep="")
    for.li<-list()
    metdat<-NULL
    anzpre<-0
    if(length(varfix)>0){
      aktvarsel<-varfix
      while(length(aktvarsel>0) & anzpre<maxanz){
        anzpre<-anzpre+1
        rsq<-sapply(aktvarsel,function(arg) summary(lm(paste(akt.for,"+",arg,sep=""),data=dataakt))$r.squared)
        aktvar<-rsq[which.max(rsq)]
        akt.for<-paste(akt.for,"+",names(aktvar),sep="")
        mod<-lm(akt.for,data=dataakt)
        metdat<-rbind(metdat,data.frame(var=names(aktvar),anz.pre=anzpre,R2=aktvar,
                                        R2.adj=summary(mod)$adj.r.squared))
        for.li<-append(for.li,list(akt.for))
        aktvarsel<-setdiff(aktvarsel,names(aktvar))
      }
    }
    aktvarsel<-varsel
    while(length(aktvarsel>0) & anzpre<maxanz){
      anzpre<-anzpre+1
      rsq<-sapply(aktvarsel,function(arg) summary(lm(paste(akt.for,"+",arg,sep=""),data=dataakt))$r.squared)
      aktvar<-rsq[which.max(rsq)]
      akt.for<-paste(akt.for,"+",names(aktvar),sep="")
      mod<-lm(akt.for,data=dataakt)
      metdat<-rbind(metdat,data.frame(var=names(aktvar),anz.pre=anzpre,R2=aktvar,
                                      R2.adj=summary(mod)$adj.r.squared))
      for.li<-c(for.li,akt.for)
      aktvarsel<-setdiff(aktvarsel,names(aktvar))
    }
    akt.li<-list(metdat)
    names(akt.li)<-paste(pop,".",index,sep="")
    res.li<-append(res.li,akt.li)
  }
}
endtime<-Sys.time()
endtime-starttime
forwarsel.li<-res.li

#####################################################################################################################
# repeated splitting sample
starttime<-Sys.time()
res.li<-list()
te.ant<-0.2
seedv<-1000000
staiter<-0
iterati<-1000
data<-rbind(dat.li[["F.con"]][,c("ID","Typ")],dat.li[["M.con"]][,c("ID","Typ")])
#####################################################################################################################
# following computation need some time
#####################################################################################################################
for(pop in c("F","M")){
  dataakt<-merge(dat.li[[paste(pop,".con",sep="")]],met.li[[paste(pop,".con",sep="")]],by="ID")
  n<-dim(dataakt)[1]
  tss<-(n-1)*var(dataakt[,tar])
  for(index in 1:length(myli)){
    varfix<-myli[[index]][[1]]
    varsel<-myli[[index]][[2]]
    metafix<-NULL
    modlfix<-list()
    metasel<-NULL
    modlsel<-list()
    set.seed(seedv)
    for(iter in (staiter+1):(staiter+iterati)){
      te.akt<-NULL
      te.akt<-c(te.akt,as.character(sample(data$ID[data$Typ==0],round(te.ant*length(data$ID[data$Typ==0]),0),replace=FALSE)))
      te.akt<-c(te.akt,as.character(sample(data$ID[data$Typ==1],round(te.ant*length(data$ID[data$Typ==1]),0),replace=FALSE)))
      te.akt<-c(te.akt,as.character(sample(data$ID[data$Typ==2],round(te.ant*length(data$ID[data$Typ==2]),0),replace=FALSE)))
      te.akt<-intersect(te.akt,as.character(dataakt$ID))
      tr.akt<-setdiff(as.character(dataakt$ID),te.akt)
      akt.for<-paste(tar,"~1",sep="")
      anzpre<-0
      if(length(varfix)>0){
        aktvarsel<-varfix
        rsq<-sapply(aktvarsel,function(arg) summary(lm(paste(akt.for,"+",arg,sep=""),data=dataakt))$r.squared)
        aktvar<-rsq[which.max(rsq)]
        akt.for<-paste(akt.for,"+",names(aktvar),sep="")
        anzpre<-anzpre+1
        aktvarsel<-setdiff(aktvarsel,names(aktvar))
        mod.te<-lm(akt.for,data=dataakt[dataakt$ID%in%tr.akt,])
        mse.te<-mean((as.numeric(predict(mod.te,dataakt[dataakt$ID%in%te.akt,]))-
                        dataakt[dataakt$ID%in%te.akt,tar])^2)
        mse.akt<-0
        while(length(aktvarsel>0) & anzpre<maxanz & mse.akt<mse.te){
          rsq<-sapply(aktvarsel,function(arg) summary(lm(paste(akt.for,"+",arg,sep=""),data=dataakt))$r.squared)
          aktvar<-rsq[which.max(rsq)]
          mod<-lm(paste(akt.for,"+",names(aktvar),sep=""),data=dataakt[dataakt$ID%in%tr.akt,])
          mse.akt<-mean((as.numeric(predict(mod,dataakt[dataakt$ID%in%te.akt,]))-
                           dataakt[dataakt$ID%in%te.akt,tar])^2)
          if(mse.akt<mse.te){
            anzpre<-anzpre+1
            akt.for<-paste(akt.for,"+",names(aktvar),sep="")
            aktvarsel<-setdiff(aktvarsel,names(aktvar))
            mod.te<-mod
            mse.te<-mse.akt
            mse.akt<-0
          }
        }
        mse.tr<-mean((as.numeric(predict(mod.te,dataakt[dataakt$ID%in%tr.akt,]))-
                        dataakt[dataakt$ID%in%tr.akt,tar])^2)
        mod<-summary(lm(akt.for,data=dataakt))
        modlfix<-append(modlfix,strsplit(akt.for,split="+",fixed=TRUE))
        metafix<-rbind(metafix,data.frame(It=iter,Anz=iterati,Seed=seedv,anz.pre=anzpre,R2=mod$r.squared,
                                          R2.adj=mod$adj.r.squared,MSE.tr=mse.tr,MSE.te=mse.te))
        akt.for<-paste(tar,"~1+",paste(varfix,collapse="+"),sep="")
        anzpre<-length(varfix)
      }
      aktvarsel<-varsel
      rsq<-sapply(aktvarsel,function(arg) summary(lm(paste(akt.for,"+",arg,sep=""),data=dataakt))$r.squared)
      aktvar<-rsq[which.max(rsq)]
      akt.for<-paste(akt.for,"+",names(aktvar),sep="")
      anzpre<-anzpre+1
      aktvarsel<-setdiff(aktvarsel,names(aktvar))
      mod.te<-lm(akt.for,data=dataakt[dataakt$ID%in%tr.akt,])
      mse.te<-mean((as.numeric(predict(mod.te,dataakt[dataakt$ID%in%te.akt,]))-
                      dataakt[dataakt$ID%in%te.akt,tar])^2)
      mse.akt<-0
      while(length(aktvarsel>0) & anzpre<maxanz & mse.akt<mse.te){
        rsq<-sapply(aktvarsel,function(arg) summary(lm(paste(akt.for,"+",arg,sep=""),data=dataakt))$r.squared)
        aktvar<-rsq[which.max(rsq)]
        mod<-lm(paste(akt.for,"+",names(aktvar),sep=""),data=dataakt[dataakt$ID%in%tr.akt,])
        mse.akt<-mean((as.numeric(predict(mod,dataakt[dataakt$ID%in%te.akt,]))-
                         dataakt[dataakt$ID%in%te.akt,tar])^2)
        if(mse.akt<mse.te){
          anzpre<-anzpre+1
          akt.for<-paste(akt.for,"+",names(aktvar),sep="")
          aktvarsel<-setdiff(aktvarsel,names(aktvar))
          mod.te<-mod
          mse.te<-mse.akt
          mse.akt<-0
        }
      }
      mse.tr<-mean((as.numeric(predict(mod.te,dataakt[dataakt$ID%in%tr.akt,]))-
                      dataakt[dataakt$ID%in%tr.akt,tar])^2)
      mod<-summary(lm(akt.for,data=dataakt))
      modlsel<-append(modlsel,strsplit(akt.for,split="+",fixed=TRUE))
      metasel<-rbind(metasel,data.frame(It=iter,Anz=iterati,Seed=seedv,anz.pre=anzpre,R2=mod$r.squared,
                                        R2.adj=mod$adj.r.squared,MSE.tr=mse.tr,MSE.te=mse.te))
    }
    akt.li<-list(list(metafix,modlfix,metasel,modlsel))
    names(akt.li)<-paste(pop,".",index,sep="")
    res.li<-append(res.li,akt.li)
  }
}
splitwdh.li<-res.li

myli<-list(list(klv,met),list(NULL,c(klv,met)))
out.lis<-list()
for(pop in c("F","M")){
  for(index in 1:length(myli)){
    meto<-forwarsel.li[[paste(pop,".",index,sep="")]]
    meto$Variable<-meto$var
    meto$Abbreviation<-meto$var
    meto<-meto[,c(1,2,5,6,3,4)]
    for(ind in 1:dim(meto)[1]){
      meto$Variable[ind]<-karmen.anno[karmen.anno$variable=="Variable",meto$var[ind]]
      meto$Abbreviation[ind]<-karmen.anno[karmen.anno$variable=="Abbreviation",meto$var[ind]]
    }
    varfix<-myli[[index]][[1]]
    varsel<-myli[[index]][[2]]
    meto$rel.H<-NA
    if(length(varfix)>0){
      modlist<-splitwdh.li[[paste(pop,".",index,sep="")]][[2]]
      anzmod<-length(modlist)
      varset<-NULL
      for(wdh in 1:anzmod){varset<-c(varset,modlist[[wdh]])}
      for(indnam in varfix){meto$rel.H[meto$var==indnam]<-sum(varset==indnam)/anzmod}
      metakt<-meto[meto$var%in%varfix,]
      meto$rel.H[meto$var%in%varfix]<-"fixed"
      out.lis<-append(out.lis,list(metakt))
    }
    if(length(varsel)>0){
      modlist<-splitwdh.li[[paste(pop,".",index,sep="")]][[4]]
      anzmod<-length(modlist)
      varset<-NULL
      for(wdh in 1:anzmod){varset<-c(varset,modlist[[wdh]])}
      for(indnam in varsel){meto$rel.H[meto$var==indnam]<-sum(varset==indnam)/anzmod}
      out.lis<-append(out.lis,list(meto))
    }
  }
}
linreg.li<-out.lis
for(index in 1:length(linreg.li)){
  linreg.li[[index]]<-linreg.li[[index]][,-1]
  colnames(linreg.li[[index]])<-c("Step","Variable","Abbreviation","R2","R2.adj","Selection Frequency")
}
#####################################################################################################################
#####################################################################################################################
#####################################################################################################################
res<-list(klv,met,cor.klch,pls.perm,pls.vali,linreg.li)
saveRDS(res,"File S7_Result.rds")
