# File S8_Output.r
# Input: File S7_Results.rds
# Output: 
#    Figure 2.png
#    Table 1.xlsx
#    Table 2.xlsx
#    Table S2_Results of bi- and multivariate association analyses.xlsx
#    Figure 3.png
#    Table S3_Results of multiple linear regression analyses.xlsx

library("openxlsx")
library("ggplot2")
library("ggrepel")
library("ggpubr")

res<-readRDS("File S7_Result.rds")

klv<-res[[1]]
met<-res[[2]]
cor.klch<-res[[3]]
pls.perm<-res[[4]]
pls.vali<-res[[5]]
linreg.li<-res[[6]]

corrp<-pls.perm[,c("pop","adj","lname","sname","Major.Pathway.Class","Specific.Pathway.Class","HMDB.ID","KEGG.ID",
                  "PC","pv","lo","up","rg.ori","rg.per","rg.pv")]
colnames(corrp)<-c("pop","adj","Variable","Abbreviation","Major pathway/class","Specific pathway/class","HMDB ID","KEGG ID",
                   "PC","pv1","lo","up","RP measured","mean RP permutated","pv2")
corrp<-corrp[order(corrp$`Abbreviation`),]
corrp.li<-list("F*"=corrp[corrp$pop=="F"&corrp$adj=="con",-(1:2)],
               "F**"=corrp[corrp$pop=="F"&corrp$adj=="all",-(1:2)],
               "M*"=corrp[corrp$pop=="M"&corrp$adj=="con",-(1:2)],
               "M**"=corrp[corrp$pop=="M"&corrp$adj=="all",-(1:2)])

#####################################################################################################################
#####################################################################################################################
#####################################################################################################################
# Figure 2
akt<-cor.klch[cor.klch$adj%in%c("con"),]
ymax<-max(c(-akt$lo,akt$up))
akt$popadj<-paste(akt$pop,".",akt$adj,sep="")
lev<-c(klv)
akt$name<-factor(akt$name,levels=lev)
levnamakt<-data.frame(x=NA)
for(ind in 1:length(klv)){
  levnamakt<-cbind(levnamakt,data.frame(x=akt$sname[akt$pop=="M"&akt$adj=="con"&akt$name==klv[ind]]))
}
levnamakt<-levnamakt[,-1]
colnames(levnamakt)<-klv
plot1<-ggplot(data=akt[akt$adj%in%c("con"),],aes(x=name,y=PC,color=pop))+
  ylim(-ymax,ymax)+
  geom_hline(aes(yintercept=0),linetype="dashed",size=1.5)+
  geom_hline(aes(yintercept=0.25),linetype="dashed",size=1)+
  geom_hline(aes(yintercept=-0.25),linetype="dashed",size=1)+
  geom_hline(aes(yintercept=0.5),linetype="dashed",size=1)+
  geom_hline(aes(yintercept=-0.5),linetype="dashed",size=1)+
  geom_errorbar(aes(ymin=lo,ymax=up,color=pop),width=.6,size=1.2,position=position_dodge(width=0.5))+
  geom_point(position=position_dodge(width=0.5),size=3)+
  labs(x=NULL,y="Correlation")+
  scale_x_discrete(breaks=lev,labels=levnamakt,limits=lev)+
  scale_color_manual(name="",breaks=c("F","M"),labels=c("Female*","Male*"),values=c("F"="lightblue","M"="lightcoral"))+
  theme(text=element_text(size=20),axis.text.x=element_text(angle=90,vjust=0.5),legend.position="top")

png(file="Figure 2.png",width=900,height=450)
plot1
dev.off()

#####################################################################################################################
#####################################################################################################################
#####################################################################################################################
# Table 1
pathnum<-data.frame(Pathways=c("All",unique(corrp$`Major pathway/class`)),
                    total=NA,S3=NA,S4=NA,S5=NA,S6=NA)
colnames(pathnum)[3:6]<-c("F*","F**","M*","M**")
pathnum$Pathways[10]<-"Unknown"
mpc<-corrp.li[[1]]$`Major pathway/class`
mpc[is.na(mpc)]<-"Unknown"
for(index in 2:10){
  pathnum[index,"total"]<-sum(mpc==pathnum$Pathways[index])
}
pathnum[1,"total"]<-sum(pathnum[2:10,"total"])
pathnum[2:9,]<-pathnum[1+order(-pathnum$total[2:9]),]
for(indnam in names(corrp.li)){
  mpc<-corrp.li[[indnam]]$`Major pathway/class`[corrp.li[[indnam]]$pv1<0.05]
  mpc[is.na(mpc)]<-"Unknown"
  for(index in 2:10){
    pathnum[index,indnam]<-sum(mpc==pathnum$Pathways[index])
  }
  pathnum[1,indnam]<-sum(pathnum[2:10,indnam])
}
write.xlsx(pathnum,file="Table 1.xlsx")

#####################################################################################################################
#####################################################################################################################
#####################################################################################################################
# Table 2

akt<-corrp.li[["F**"]][,c("Abbreviation","PC","lo","up")]
colnames(akt)[1]<-"Variables"
akt<-akt[order(akt$PC),]
akt1<-cbind(Females=1:10,akt[1:10,])
akt1[,3:5]<-round(akt1[,3:5],2)
akt1$"r(95% CIs)"<-paste(akt1$PC," (",akt1$lo,", ",akt1$up,")",sep="")
akt.Fneg<-akt1[,c(1,2,6)]
akt1<-cbind(Females=1:10,akt[dim(akt)[1]:(dim(akt)[1]-9),])
akt1[,3:5]<-round(akt1[,3:5],2)
akt1$"r(95% CIs)"<-paste(akt1$PC," (",akt1$lo,", ",akt1$up,")",sep="")
akt.Fpos<-akt1[,c(1,2,6)]
akt<-corrp.li[["M**"]][,c("Abbreviation","PC","lo","up")]
colnames(akt)[1]<-"Variables"
akt<-akt[order(akt$PC),]
akt1<-cbind(Males=1:10,akt[1:10,])
akt1[,3:5]<-round(akt1[,3:5],2)
akt1$"r(95% CIs)"<-paste(akt1$PC," (",akt1$lo,", ",akt1$up,")",sep="")
akt.Mneg<-akt1[,c(1,2,6)]
akt1<-cbind(Males=1:10,akt[dim(akt)[1]:(dim(akt)[1]-9),])
akt1[,3:5]<-round(akt1[,3:5],2)
akt1$"r(95% CIs)"<-paste(akt1$PC," (",akt1$lo,", ",akt1$up,")",sep="")
akt.Mpos<-akt1[,c(1,2,6)]
write.xlsx(list("Fpos"=akt.Fpos,"Fneg"=akt.Fneg,"Mpos"=akt.Mpos,"Mneg"=akt.Mneg),file="Table 2.xlsx")

#####################################################################################################################
#####################################################################################################################
#####################################################################################################################
# Table S2 Results of bi- and multivariate association analyses
names(corrp.li)<-c("F_adj1","F_adj2","M_adj1","M_adj2")
write.xlsx(corrp.li,file="Table S2_Results of bi- and multivariate association analyses.xlsx",startRow=4)

#####################################################################################################################
#####################################################################################################################
#####################################################################################################################
# Figure 3
nam<-met
akt<-pls.perm
akt$rg.pv[akt$rg.pv==0]<-min(c(akt$rg.pv)[c(akt$rg.pv)>0])/2
ymax<--log10(min(akt$rg.pv))
ymaxcor<-max(c(-akt$lo,akt$up))
akt$Major.Pathway.Class[is.na(akt$Major.Pathway.Class)]<-"Unknown"
akt$Abbreviation.2.short<-akt$sname
akt$popadj<-paste(akt$pop,".",akt$adj,sep="")
akt$popadj<-factor(akt$popadj,levels=c("F.con","F.all","M.con","M.all"))

akt$logpv<--log10(akt$rg.pv)
akt$labelgr<-0
akt$labelgr[akt$Major.Pathway.Class!="Unknown"&akt$popadj=="F.con"&akt$PC<(-0.25)]<-5.1
akt$labelgr[akt$Major.Pathway.Class!="Unknown"&akt$popadj=="F.con"&akt$PC>0.25&akt$PC<=0.29]<-5.2
akt$labelgr[akt$Major.Pathway.Class!="Unknown"&akt$popadj=="F.con"&akt$PC>0.29&akt$PC<=0.33]<-5.3
akt$labelgr[akt$Major.Pathway.Class!="Unknown"&akt$popadj=="F.con"&akt$PC>0.33&akt$PC<=0.37]<-5.4
akt$labelgr[akt$Major.Pathway.Class!="Unknown"&akt$popadj=="F.con"&akt$PC>0.37]<-5.5

akt$labelgr[akt$Major.Pathway.Class!="Unknown"&akt$popadj=="M.con"&akt$rg.pv<0.05&akt$PC<(-0.202)]<-5.1
akt$labelgr[akt$Major.Pathway.Class!="Unknown"&akt$popadj=="M.con"&akt$rg.pv<0.05&abs(akt$PC)<=0.202]<-5.3
akt$labelgr[akt$Major.Pathway.Class!="Unknown"&akt$popadj=="M.con"&akt$rg.pv<0.05&akt$PC>0.202]<-5.2

popadjnam<-c("F.all"="F**","F.con"="F*","M.all"="M**","M.con"="M*")
plot.li<-list()
for(popadj in c("F.con","M.con")){
  plot1<-ggplot(data=akt[akt$popadj%in%popadj,],aes(x=PC,y=logpv,color=Major.Pathway.Class))+
    ylim(0,ymax)+
    xlim(-0.6,0.6)+
    geom_hline(aes(yintercept=-log10(0.05)),linetype="dashed",size=1)+
    geom_vline(aes(xintercept=-0.25),linetype="dashed",size=1)+
    geom_vline(aes(xintercept= 0.25),linetype="dashed",size=1)+
    ggtitle(popadjnam[popadj])+
    geom_text_repel(mapping=aes(label=Abbreviation.2.short),size=5,direction="both",nudge_x=-1,
                    segment.linetype=3,segment.size=0.5,data=subset(akt[akt$popadj%in%popadj,],labelgr==5.1))+
    geom_text_repel(mapping=aes(label=Abbreviation.2.short),size=5,direction="both",nudge_x=1,nudge_y=-0.5,
                    segment.linetype=3,segment.size=0.5,data=subset(akt[akt$popadj%in%popadj,],labelgr==5.2))+
    geom_text_repel(mapping=aes(label=Abbreviation.2.short),size=5,direction="both",nudge_x=-0.3,nudge_y=0.5,
                    segment.linetype=3,segment.size=0.5,data=subset(akt[akt$popadj%in%popadj,],labelgr==5.3))+
    geom_text_repel(mapping=aes(label=Abbreviation.2.short),size=5,direction="both",nudge_x=1,nudge_y=1,
                    segment.linetype=3,segment.size=0.5,data=subset(akt[akt$popadj%in%popadj,],labelgr==5.4))+
    geom_text_repel(mapping=aes(label=Abbreviation.2.short),size=5,direction="both",nudge_x=-0.3,nudge_y=1,
                    segment.linetype=3,segment.size=0.5,data=subset(akt[akt$popadj%in%popadj,],labelgr==5.5))+
    geom_point(size=3)+
    labs(x="Partial correlation",y="Contribution to PLS model")+
    scale_color_manual(name="",breaks=unique(akt$Major.Pathway.Class),
                       values=c("Amino acid metabolism"="#2C4D75",
                                "Xenobiotics and related metabolism"="#70AD47",
                                "Mammalian-microbial cometabolism"="#ED7D31",
                                "Cofactors and vitamins metabolism"="#006600",
                                "Carbohydrate metabolism"="#FFC000",
                                "Lipid metabolism"="#9E480E",
                                "Unknown"="#000000",
                                "Energy metabolism"="#5B9BD5",
                                "Nucleotide metabolism"="#660066"))+
    theme(text=element_text(size=20),legend.position="none")
  plot.li<-append(plot.li,list(plot1))
}
png(file="Figure 3.png",width=900,height=1200)
ggarrange(plotlist=plot.li,ncol=1,nrow=2)
dev.off()

#####################################################################################################################
#####################################################################################################################
#####################################################################################################################
# Table S3_Results of multiple linear regression analyses
names(linreg.li)<-c("F_Approach 1","F_Approach 2","F_Approach 3","M_Approach 1","M_Approach 2","M_Approach 3")
write.xlsx(linreg.li,file="Table S3_Results of multiple linear regression analyses.xlsx",startRow=5)
